import unittest

from butter.mas.api import HttpClient


class TestClientApiMethods(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.client = None

    def testGetAvailableHandlers(self):
        self.assertIsNotNone(self.client.getAvailableHandlers())

    def testGetAvailableAnimations(self):
        self.assertIsNotNone(self.client.getAvailableAnimations())

    def testGetAvailableSounds(self):
        self.assertIsNotNone(self.client.getAvailableSounds())

    def testGetAvailableMotorRegisters(self):
        self.assertIsNotNone(self.client.getAvailableMotorRegisters('base'))

    def testGetMotorRegister(self):
        self.assertIsNotNone(self.client.getMotorRegister('base', 'goal_position'))

    def testGetMotorRegisterRange(self):
        self.assertIsNotNone(self.client.getMotorRegisterRange('base', 'goal_position'))

    def testSetMotorRegister(self):
        self.assertIsNotNone(self.client.setMotorRegister('base', 'goal_position', '2048'))

    def testMoveMotorToPosition(self):
        self.assertIsNotNone(self.client.moveMotorToPosition('base', '2048', '2'))

    def testMoveMotorInTime(self):
        self.assertIsNotNone(self.client.moveMotorInTime('base', '2048', '12'))

    def testMoveMotorInDirection(self):
        self.assertIsNotNone(self.client.moveMotorToPosition('base', 'left', '2'))

    # def testMoveMotorInSteps(self):
    #     self.assertIsNotNone(self.client.moveMotorInSteps('base', '2048', '2'))

    def testPlayAnimation(self):
        self.assertIsNotNone(self.client.playAnimation('welcome'))

    def testPauseAnimation(self):
        self.assertIsNotNone(self.client.pauseAnimation())

    def testResumeAnimation(self):
        self.assertIsNotNone(self.client.resumeAnimation())

    def testStopAnimation(self):
        self.assertIsNotNone(self.client.stopAnimation())

    def testPlayAudio(self):
        self.assertIsNotNone(self.client.playAudio('hello'))

    def testPauseAudio(self):
        self.assertIsNotNone(self.client.pauseAudio())

    def testResumeAudio(self):
        self.assertIsNotNone(self.client.resumeAudio())

    def testStopAudio(self):
        self.assertIsNotNone(self.client.stopAudio())



from butter.mas.clients.client_factory import ClientFactory
from butter.mas.clients.client_http import HttpClient
from butter.mas.clients.client_tcp import TcpClient
from butter.mas.clients.client_udp import UdpClient

from .environment import __version__
from butter.mas.utils.version_utils import VersionValidations
from butter.mas.utils.response_parser import ResponseParser

""" Exposes Butter MAS Python API """



import unittest

from butter.mas.api import VersionValidations


class TestVersionValidationsMethods(unittest.TestCase):

    def testGetLatestAppVersion(self):
        self.assertIsNotNone(VersionValidations._getLatestAppVersion())

    def testCheckForUpdates(self):
        self.assertFalse(VersionValidations.checkForUpdates())

    def testGetFirmwareVersions(self):
        self.assertIsNotNone(VersionValidations._getFirmwareVersions('localhost'))

    def testCheckCompatibility(self):
        self.assertTrue(VersionValidations.checkCompatibility('localhost'))



from butter.mas.api import HttpClient
from butter.mas.tests.clients.client_test import TestClientApiMethods


class TestHttpClientApiMethods(TestClientApiMethods):

    @classmethod
    def setUpClass(self):
        self.client = HttpClient('localhost')



from butter.mas.api import TcpClient
from butter.mas.tests.clients.client_test import TestClientApiMethods


class TestTcpClientApiMethods(TestClientApiMethods):

    @classmethod
    def setUpClass(self):
        self.client = TcpClient('localhost')



from butter.mas.environment import app_name, __version__
from packaging.version import Version
import sys
import requests
from requests.exceptions import HTTPError


class VersionValidations:
    """ Version and compatibility validations """

    @staticmethod
    def _getLatestAppVersion() -> str:
        version = None
        response = requests.get('https://pypi.org/pypi/{}/json'.format(app_name))

        if response:
            app_info = response.json()
            if app_info and app_info['info'] and app_info['info']['version']:
                version = app_info['info']['version']

        return version

    @staticmethod
    def _getFirmwareVersions(ip, port=3000) -> dict:
        versions = None
        response = None

        try:
            response = requests.get('http://{}:{}/api/information/versions'.format(ip, port))

            # If the response was successful, no Exception will be raised
            response.raise_for_status()
        except HTTPError as http_err:
            pass
            # print('HTTP error occurred while trying to fetch firmware version: {}'.format(http_err), file=sys.stderr)
        except Exception as error:
            pass
            # print('Some error occurred while trying to fetch firmware version: {}'.format(error), file=sys.stderr)
        else:
            versions = response.json()

        return versions

    @staticmethod
    def checkForUpdates() -> bool:
        """checks if this package is up to date
        
        Returns:
            boolean: true, if this package is up to date, otherwise, false
        """
        has_update = False
        latest_version = VersionValidations._getLatestAppVersion()

        if latest_version is None:
            print('Update check failed!', file=sys.stderr)
            print('Please check for updates manually at https://pypi.org/project/{}/'.format(app_name), file=sys.stderr)
        elif Version(__version__) < Version(latest_version):
            print('Update available! your version: {}, latest: {}'.format(__version__, latest_version))
            print('Execute `pip install -U {}` to update'.format(app_name))
            has_update = True
        else:
            print('Up to Date!')

        return has_update

    @staticmethod
    def checkCompatibility(ip) -> bool:
        """checks that this package is compatible with the robot that listening on the given ip
        
        Args:
            ip (str): robot's ip address
        
        Returns:
            boolean: true, if this package and the robot firmware are compatible, otherwise, false
        """

        is_compatible = False
        firmware_version = VersionValidations._getFirmwareVersions(ip)

        if firmware_version is None or firmware_version.get('software', {}).get('mas'):
            print('Compatibility check failed!', file=sys.stderr)
        else:
            if __version__.split('.')[0] != firmware_version.get('software', {}).get('mas').split('.')[0]:
                print('This software is not compatible with {}'.format(ip))
                print('Please update your robot firmware')
            else:
                print('This software is compatible with {}'.format(ip))
                is_compatible = False

        return is_compatible



from .packet_http import HttpPacket
from .packet_tcp import TcpPacket
from .packet_udp import UdpPacket
from .packet import Packet


class PacketFactory:
    """ Packet factory for different types of protocols """

    def getPacket(self, ip, port, query, protocol="http") -> Packet:
        """Creates new packet
        
        Arguments:
            ip (str): robot IP
            port (int): robot port
            query (str): packet payload
            protocol (str, optional): communication protocol. defaults to "http".
        
        Returns:
            Packet: data packet
        """
        if protocol == "http":
            return HttpPacket(ip, port, query)
        elif protocol == "tcp":
            return TcpPacket(ip, port, query)
        elif protocol == "udp":
            return UdpPacket(ip, port, query)
        else:
            return None

    def getPacketClass(self, protocol="http"):
        """Get packet class
        
        Args:
            protocol (str, optional): communication protocol. defaults to "http".
        
        Returns:
            Packet: data packet
        """
        if protocol == "http":
            return HttpPacket
        elif protocol == "tcp":
            return TcpPacket
        elif protocol == "udp":
            return UdpPacket
        else:
            return None



import unittest

from butter.mas.tests.utils.version_validations_test import TestVersionValidationsMethods
from butter.mas.tests.clients.client_http_test import TestHttpClientApiMethods
from butter.mas.tests.clients.client_tcp_test import TestTcpClientApiMethods
from butter.mas.tests.clients.client_udp_test import TestUdpClientApiMethods
from butter.mas.tests.packets.packet_http_test import TestHttpPacketMethods
from butter.mas.tests.packets.packet_tcp_test import TestTcpPacketMethods
from butter.mas.tests.packets.packet_udp_test import TestUdpPacketMethods
from butter.mas.tests.packets.packet_builder_test import TestPacketBuilderMethods


def suite():
    suite = unittest.TestSuite()
    loader = unittest.TestLoader()
    testCases = (TestVersionValidationsMethods,
                 TestPacketBuilderMethods,
                 TestHttpPacketMethods,
                 TestTcpPacketMethods,
                 TestUdpPacketMethods,
                 TestHttpClientApiMethods,
                 TestTcpClientApiMethods,
                 TestUdpClientApiMethods)

    for testClass in testCases:
        tests = loader.loadTestsFromTestCase(testClass)
        suite.addTests(tests)

    return suite


if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    # runner.run(suite()) # avoid running test twice (for some reason test run after import -?!-)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

environment = dict()
with open("butter/mas/environment.py") as f:
    exec(f.read(), environment)

with open('README.md', encoding="utf8") as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()

# install_requires = list()
# with open('requirements.txt', 'r', encoding='utf-8') as f:
#     for line in f.readlines():
#         install_requires.append(line.split("=")[0].strip("<~>"))

setup(
    name=environment['app_name'],
    version=environment['__version__'],
    description='Python Client API for Butter MAS platform',
    long_description_content_type='text/markdown',
    long_description=readme,
    author='Benny Megidish',
    author_email='bennymegk@gmail.com',
    url='https://github.com/bennymeg/Butter.MAS.PythonAPI',
    install_requires=['requests', 'packaging'],
    extras_requires={"dev": ['twine']},
    # license=license,
    license='Apache-2.0',
    packages=find_packages(exclude=('tests', 'docs')),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'Operating System :: OS Independent',
        'Topic :: Scientific/Engineering :: Human Machine Interfaces',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
    ],
)

''' how to deploy '''
# python setup.py sdist bdist_wheel
# python -m twine upload --repository-url https://test.pypi.org/legacy/ dist/*
# pip install --index-url https://test.pypi.org/simple/ --no-deps butter.mas-api

# python -m twine upload dist/*
# pip install -U butter.mas-api

''' how to generate and serve documentation '''
# cd sphinx
# ./serve
# update tree.json file if a new file was added (the first file on the list is the default one)



from requests import Response


class RobotResponse:
    """ *** This Type should be updated together with MAS#ResponseBuilder *** """

    executed: bool
    result: str


class ResponseParser:
    """ Parses API response data into a typed aware dictionary object """

    @staticmethod
    def parse(response: Response) -> RobotResponse:
        return response.json()



import unittest
from butter.mas.packets.packet_factory import PacketFactory


class TestTcpPacketMethods(unittest.TestCase):

    def testSend(self):
        packetFactory = PacketFactory()
        packet = packetFactory.getPacket('localhost', 5050, 'cmd/json/help', protocol='tcp')
        self.assertIsNotNone(packet.send())


if __name__ == '__main__':
    unittest.main()
